
import { ExtensionsRegistry } from 'vscode/vscode/vs/workbench/services/extensions/common/extensionsRegistry';
import { localize } from 'vscode/vscode/vs/nls';
import { launchSchemaId } from '@codingame/monaco-vscode-37e80bf5-92f5-5e4c-8b6d-98e0bb89dbef-common/vscode/vs/workbench/services/configuration/common/configuration';
import { inputsSchema } from '@codingame/monaco-vscode-2448342b-297f-5823-9f41-f18eef0f6ee8-common/vscode/vs/workbench/services/configurationResolver/common/configurationResolverSchema';
import { Disposable } from 'vscode/vscode/vs/base/common/lifecycle';
import { Extensions } from 'vscode/vscode/vs/workbench/services/extensionManagement/common/extensionFeatures';
import { SyncDescriptor } from 'vscode/vscode/vs/platform/instantiation/common/descriptors';
import { Registry } from 'vscode/vscode/vs/platform/registry/common/platform';

const debuggersExtPoint = ExtensionsRegistry.registerExtensionPoint({
    extensionPoint: 'debuggers',
    defaultExtensionKind: ['workspace'],
    jsonSchema: {
        description: ( localize(5580, 'Contributes debug adapters.')),
        type: 'array',
        defaultSnippets: [{ body: [{ type: '' }] }],
        items: {
            additionalProperties: false,
            type: 'object',
            defaultSnippets: [{ body: { type: '', program: '', runtime: '' } }],
            properties: {
                type: {
                    description: ( localize(5581, "Unique identifier for this debug adapter.")),
                    type: 'string'
                },
                label: {
                    description: ( localize(5582, "Display name for this debug adapter.")),
                    type: 'string'
                },
                program: {
                    description: ( localize(
                        5583,
                        "Path to the debug adapter program. Path is either absolute or relative to the extension folder."
                    )),
                    type: 'string'
                },
                args: {
                    description: ( localize(5584, "Optional arguments to pass to the adapter.")),
                    type: 'array'
                },
                runtime: {
                    description: ( localize(
                        5585,
                        "Optional runtime in case the program attribute is not an executable but requires a runtime."
                    )),
                    type: 'string'
                },
                runtimeArgs: {
                    description: ( localize(5586, "Optional runtime arguments.")),
                    type: 'array'
                },
                variables: {
                    description: ( localize(
                        5587,
                        "Mapping from interactive variables (e.g. ${action.pickProcess}) in `launch.json` to a command."
                    )),
                    type: 'object'
                },
                initialConfigurations: {
                    description: ( localize(5588, "Configurations for generating the initial \'launch.json\'.")),
                    type: ['array', 'string'],
                },
                languages: {
                    description: ( localize(
                        5589,
                        "List of languages for which the debug extension could be considered the \"default debugger\"."
                    )),
                    type: 'array'
                },
                configurationSnippets: {
                    description: ( localize(5590, "Snippets for adding new configurations in \'launch.json\'.")),
                    type: 'array'
                },
                configurationAttributes: {
                    description: ( localize(5591, "JSON schema configurations for validating \'launch.json\'.")),
                    type: 'object'
                },
                when: {
                    description: ( localize(
                        5592,
                        "Condition which must be true to enable this type of debugger. Consider using 'shellExecutionSupported', 'virtualWorkspace', 'resourceScheme' or an extension-defined context key as appropriate for this."
                    )),
                    type: 'string',
                    default: ''
                },
                hiddenWhen: {
                    description: ( localize(
                        5593,
                        "When this condition is true, this debugger type is hidden from the debugger list, but is still enabled."
                    )),
                    type: 'string',
                    default: ''
                },
                deprecated: {
                    description: ( localize(5594, "Optional message to mark this debug type as being deprecated.")),
                    type: 'string',
                    default: ''
                },
                windows: {
                    description: ( localize(5595, "Windows specific settings.")),
                    type: 'object',
                    properties: {
                        runtime: {
                            description: ( localize(5596, "Runtime used for Windows.")),
                            type: 'string'
                        }
                    }
                },
                osx: {
                    description: ( localize(5597, "macOS specific settings.")),
                    type: 'object',
                    properties: {
                        runtime: {
                            description: ( localize(5598, "Runtime used for macOS.")),
                            type: 'string'
                        }
                    }
                },
                linux: {
                    description: ( localize(5599, "Linux specific settings.")),
                    type: 'object',
                    properties: {
                        runtime: {
                            description: ( localize(5600, "Runtime used for Linux.")),
                            type: 'string'
                        }
                    }
                },
                strings: {
                    description: ( localize(5601, "UI strings contributed by this debug adapter.")),
                    type: 'object',
                    properties: {
                        unverifiedBreakpoints: {
                            description: ( localize(
                                5602,
                                "When there are unverified breakpoints in a language supported by this debug adapter, this message will appear on the breakpoint hover and in the breakpoints view. Markdown and command links are supported."
                            )),
                            type: 'string'
                        }
                    }
                }
            }
        }
    }
});
const breakpointsExtPoint = ExtensionsRegistry.registerExtensionPoint({
    extensionPoint: 'breakpoints',
    jsonSchema: {
        description: ( localize(5603, 'Contributes breakpoints.')),
        type: 'array',
        defaultSnippets: [{ body: [{ language: '' }] }],
        items: {
            type: 'object',
            additionalProperties: false,
            defaultSnippets: [{ body: { language: '' } }],
            properties: {
                language: {
                    description: ( localize(5604, "Allow breakpoints for this language.")),
                    type: 'string'
                },
                when: {
                    description: ( localize(
                        5605,
                        "Condition which must be true to enable breakpoints in this language. Consider matching this to the debugger when clause as appropriate."
                    )),
                    type: 'string',
                    default: ''
                }
            }
        }
    }
});
const presentationSchema = {
    type: 'object',
    description: ( localize(
        5606,
        "Presentation options on how to show this configuration in the debug configuration dropdown and the command palette."
    )),
    properties: {
        hidden: {
            type: 'boolean',
            default: false,
            description: ( localize(
                5607,
                "Controls if this configuration should be shown in the configuration dropdown and the command palette."
            ))
        },
        group: {
            type: 'string',
            default: '',
            description: ( localize(
                5608,
                "Group that this configuration belongs to. Used for grouping and sorting in the configuration dropdown and the command palette."
            ))
        },
        order: {
            type: 'number',
            default: 1,
            description: ( localize(
                5609,
                "Order of this configuration within a group. Used for grouping and sorting in the configuration dropdown and the command palette."
            ))
        }
    },
    default: {
        hidden: false,
        group: '',
        order: 1
    }
};
const defaultCompound = { name: 'Compound', configurations: [] };
const launchSchema = {
    id: launchSchemaId,
    type: 'object',
    title: ( localize(5610, "Launch")),
    allowTrailingCommas: true,
    allowComments: true,
    required: [],
    default: { version: '0.2.0', configurations: [], compounds: [] },
    properties: {
        version: {
            type: 'string',
            description: ( localize(5611, "Version of this file format.")),
            default: '0.2.0'
        },
        configurations: {
            type: 'array',
            description: ( localize(
                5612,
                "List of configurations. Add new configurations or edit existing ones by using IntelliSense."
            )),
            items: {
                defaultSnippets: [],
                'type': 'object',
                oneOf: []
            }
        },
        compounds: {
            type: 'array',
            description: ( localize(
                5613,
                "List of compounds. Each compound references multiple configurations which will get launched together."
            )),
            items: {
                type: 'object',
                required: ['name', 'configurations'],
                properties: {
                    name: {
                        type: 'string',
                        description: ( localize(
                            5614,
                            "Name of compound. Appears in the launch configuration drop down menu."
                        ))
                    },
                    presentation: presentationSchema,
                    configurations: {
                        type: 'array',
                        default: [],
                        items: {
                            oneOf: [{
                                    enum: [],
                                    description: ( localize(5615, "Please use unique configuration names."))
                                }, {
                                    type: 'object',
                                    required: ['name'],
                                    properties: {
                                        name: {
                                            enum: [],
                                            description: ( localize(
                                                5614,
                                                "Name of compound. Appears in the launch configuration drop down menu."
                                            ))
                                        },
                                        folder: {
                                            enum: [],
                                            description: ( localize(5616, "Name of folder in which the compound is located."))
                                        }
                                    }
                                }]
                        },
                        description: ( localize(
                            5617,
                            "Names of configurations that will be started as part of this compound."
                        ))
                    },
                    stopAll: {
                        type: 'boolean',
                        default: false,
                        description: ( localize(
                            5618,
                            "Controls whether manually terminating one session will stop all of the compound sessions."
                        ))
                    },
                    preLaunchTask: {
                        type: 'string',
                        default: '',
                        description: ( localize(5619, "Task to run before any of the compound configurations start."))
                    }
                },
                default: defaultCompound
            },
            default: [
                defaultCompound
            ]
        },
        inputs: inputsSchema.definitions.inputs
    }
};
class DebuggersDataRenderer extends Disposable {
    constructor() {
        super(...arguments);
        this.type = 'table';
    }
    shouldRender(manifest) {
        return !!manifest.contributes?.debuggers;
    }
    render(manifest) {
        const contrib = manifest.contributes?.debuggers || [];
        if (!contrib.length) {
            return { data: { headers: [], rows: [] }, dispose: () => { } };
        }
        const headers = [
            ( localize(5620, "Name")),
            ( localize(5621, "Type")),
        ];
        const rows = ( contrib.map(d => {
            return [
                d.label ?? '',
                d.type
            ];
        }));
        return {
            data: {
                headers,
                rows
            },
            dispose: () => { }
        };
    }
}
( Registry.as(Extensions.ExtensionFeaturesRegistry)).registerExtensionFeature({
    id: 'debuggers',
    label: ( localize(5622, "Debuggers")),
    access: {
        canToggle: false
    },
    renderer: ( new SyncDescriptor(DebuggersDataRenderer)),
});

export { breakpointsExtPoint, debuggersExtPoint, launchSchema, presentationSchema };
